`timescale 1ns / 1ps
`define CLK_PERIOD 20

module ad7606_uart_tb();

	xsGSR xsGSR_INST(.GSR(1'b1));
	xsPWR xsPWR_INST(.PUR(1'b1));

    reg Clk;
	reg Reset_n;
	
	reg Dout_A;
	reg Dout_B;
	
	wire ad7606_cs_n_o;
	wire ad7606_rd_n_o;
	wire [2:0]ad7606_os_o;
	wire ad7606_convst_o;
	wire ad7606_db15;
	wire [5:0]ad7606_db14_9;
	wire [6:0]ad7606_db6_0;
	wire ad7606_frstdata;
		
	wire Rs232_Tx;
	wire led;
	  
	initial Clk = 1;
	always #10 Clk = ~Clk;
	
	reg Clk_0;
 	initial Clk_0 = 1;
	always #20 Clk_0 = ~Clk_0;
	wire Rs232_Tx_0;
	
	ad7606_uart ad7606_uart(
		.clk25m(Clk_0),
		.Reset_n(Reset_n),
		.Rs232_Tx(Rs232_Tx),
		.Rs232_Rx(Rs232_Tx_0),
		.led(led),
		.ad7606_cs_n_o(ad7606_cs_n_o),
		.ad7606_rd_n_o(ad7606_rd_n_o),
		.ad7606_os_o(ad7606_os_o),
		.ad7606_convst_o(ad7606_convst_o),
		.ad7606_db15(ad7606_db15),
		.ad7606_db14_9(ad7606_db14_9),
		.ad7606_db6_0(ad7606_db6_0),
		.ad7606_frstdata(ad7606_frstdata),
		
		.Dout_A(Dout_A),
		.Dout_B(Dout_B)
	);
	
	reg [15:0]cnt_data;
	reg cnten;
	
	always@(posedge Clk_0)
	if(cnten)begin
		cnt_data <= cnt_data +1;
		Dout_A <= cnt_data[0];
		Dout_B <= cnt_data[2];
		if(cnt_data == 1300)begin
			cnt_data <= 0;
			Dout_A <= 0;
			Dout_B <= 0;
		end	
	end
	else begin
		cnt_data <= 0;
		Dout_A <= 0;
		Dout_B <= 0;
	end
	
	reg uart_send_en;
	reg [7:0]uart_tx_data;
	wire uart_tx_done;
	
   initial begin
		Reset_n = 0;
		cnten =0;
		#201;
		Reset_n = 1;
		#200;
		uart_send_en = 1'b0;
		uart_tx_data = 8'd0;
		#(`CLK_PERIOD*10+1);
		cnten =1;
		test_inst;
		#(`CLK_PERIOD*4000000);


		#400000;
		cnten =0;
		
		#2000;
		$stop;	
	end
   
task test_inst;
	begin
		//采样频率,仿真串口发送55 A5 03 00 00 00 FF F0
		uart_tx_data = 8'h55;
		uart_send_en = 1'b1;//-----发送55
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hA5;
		uart_send_en = 1'b1;//-----发送A5
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h03;
		uart_send_en = 1'b1;//-----发送03
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		
		uart_tx_data = 8'h40;
		uart_send_en = 1'b1;//-----发送
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hF0;
		uart_send_en = 1'b1;//-----发送F0
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
			
		//采集样本量,仿真串口发送55 A5 02 00 00 10 00 F0
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h55;
		uart_send_en = 1'b1;//-----发送55
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hA5;
		uart_send_en = 1'b1;//-----发送A5
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h02;
		uart_send_en = 1'b1;//-----发送02
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		
		uart_tx_data = 8'h01;
		uart_send_en = 1'b1;//-----发送01repeat(512)
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hF0;
		uart_send_en = 1'b1;//-----发送F0
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		
		//采集通道设置,仿真串口发送55 A5 01 00 00 00 01 F0
		
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h55;
		uart_send_en = 1'b1;//-----发送55
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hA5;
		uart_send_en = 1'b1;//-----发送A5
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h01;
		uart_send_en = 1'b1;//-----发送01
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		
		uart_tx_data = 8'h01;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hF0;
		uart_send_en = 1'b1;//-----发送F0
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		
		//采样使能,仿真串口发送55 A5 00 00 00 00 00 F0
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h55;
		uart_send_en = 1'b1;//-----发送55
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hA5;
		uart_send_en = 1'b1;//-----发送A5
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		//
		uart_tx_data = 8'h00;
		uart_send_en = 1'b1;//-----发送00
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		#(`CLK_PERIOD*10+1);
		uart_tx_data = 8'hF0;
		uart_send_en = 1'b1;//-----发送F0
		#(`CLK_PERIOD);
		uart_send_en = 1'b0;
		@(posedge uart_tx_done);
		
		#(`CLK_PERIOD*10+1);
	end
endtask	

	uart_byte_tx uart_byte_tx_cmd(
		.Clk (Clk),//50M时钟输入,
		.Rst_n (Reset_n),//模块复位,
		.data_byte (uart_tx_data ),//待传输8bit数据,
		.send_en (uart_send_en ),//发送使能,
		.baud_set (4'd4),//串口波特率设置
		.Rs232_Tx(Rs232_Tx_0),//Rs232输出信号,
		.Tx_Done (uart_tx_done),//一次发送数据完成标志,
		.uart_state() //发送数据状态
	);
	
endmodule
